/**
   @file gl_drawing_area_if.h
   @brief 描画領域クラス定義用ヘッダ

   @author tadah_fussy
   @date 2018/02/24 DrawingMethod.h から分離
   @date 2021/04/02 文字コードを UTF-8 に変更
**/
#ifndef GRAPHIC_LIBRARY_DRAWING_AREA_IF_20210402_H
#define GRAPHIC_LIBRARY_DRAWING_AREA_IF_20210402_H

#include <common/math_library.h>
#include <common/coord.h>
#include <common/matrix.h>

#include "gl_color.h"

namespace tadah_fussy
{
  namespace gl
  {
    namespace canvas
    {
      /**
         @brief 描画オブジェクト用基底クラス
      **/
      class DrawingArea_IF
      {
      public:

        /// @brief 指定した座標の RGB コードを取得する
        ///
        /// 座標が不正な場合など、失敗した場合の処理は派生クラスの実装に依存する。
        ///
        /// @param c RGB コードを取得する座標値
        /// @param rgb 取得した RGB コード
        ///
        /// @return 派生クラスの実装に依存(通常、取得に成功したら true を返す)
        virtual bool point( Coord< int > c, RGB* rgb ) const = 0;

        /// @brief 指定した座標に RGB コードをセットする
        ///
        /// 座標が不正な場合など、失敗した場合の処理は派生クラスの実装に依存する。
        ///
        /// @param c RGB コードをセットする座標値
        /// @param rgb セットする RGB コード
        ///
        /// @return 派生クラスの実装に依存(通常、描画に成功したら true を返す)
        virtual bool pset( Coord< int > c, RGB rgb ) = 0;

        /// @brief 画像サイズを返す
        ///
        /// @return 画像サイス
        virtual Coord< int > size() const = 0;

        /// @brief 画像サイズを変更する
        ///
        /// @param sz 変更する画像サイズ
        virtual void resize( Coord< int > sz ) = 0;

        /// @brief 仮想デストラクタ (何もしない)
        virtual ~DrawingArea_IF() {}
      };

    }

  }

}

#endif
