#include "gl_g_pixel_op.h"

namespace tadah_fussy
{
  namespace gl
  {
    namespace pen
    {
      using coord_lib::Gradient;
      using coord_lib::Intercept;

      using canvas::DrawingArea_IF;

      //using shape::circle::ANGLE_MAX;

      /**************************************************************
        点描画用クラス
      **************************************************************/

      /*********************************************************
        GLinePatternSet : ライン・パターンに従った点描画クラス
      **********************************************************/

      /*
        GLinePatternSet::operator() : ライン・パターンに従った点描画処理
      */
      bool GLinePatternSet::operator()( DrawingArea_IF& draw, Coord< int > c )
      {
        bool b = false;
        if ( ( pattern_ >> ( PATTERN_SIZE - index_ - 1 ) & 1 ) )
          b = ( *pixelOp_ )( draw, c );

        index_ = ( index_ + 1 ) % PATTERN_SIZE;

        return( b );
      }

      /****************************************************
        GBoxPatternSet : 矩形パターンに従った点描画クラス
      *****************************************************/

      /*
        GBoxPatternSet::operator() : 矩形パターンに従った点描画処理
      */
      bool GBoxPatternSet::operator()( DrawingArea_IF& draw, Coord< int > c )
      {
        Coord< int > pSize( pattern_.size() );
        if ( pSize.x <= 0 || pSize.y <= 0 ) return( false );
        Coord< int > pc( c.x % pattern_.size().x, c.y % pattern_.size().y );
        if ( pc.x < 0 ) pc.x += pattern_.size().x;
        if ( pc.y < 0 ) pc.y += pattern_.size().y;

        RGB fg;
        if ( ! pattern_.point( pc, &fg ) )
          return( false );
        gRGBOp_.setRGB( fg );

        return( gRGBOp_( draw, c ) );
      }

      /********************************
        GNotSet : 色反転描画クラス
      *********************************/

      /*
        GNotSet::operator() : 色反転点描画処理
      */
      bool GNotSet::operator()( DrawingArea_IF& draw, Coord<int> c )
      {
        RGB rgb;

        if ( ! draw.point( c, &rgb ) )
          return( false );

        return( draw.pset( c, ~rgb ) );
      }

      /**********************************************
        GPixelCopy : 描画領域からピクセルをコピーする
      ***********************************************/

      /*
        GPixelCopy::operator() : 描画領域 src から位置 c のピクセルをコピーする

        位置はコピー元の原点を基準とする。
      */
      bool GPixelCopy::operator()( DrawingArea_IF& src, Coord< int > c )
      {
        RGB rgb;

        if ( ! src.point( c, &rgb ) )
          return( false );

        c -= offset_;
        return( dest_.pset( c, rgb ) );
      }

      /*****************************************
        CrossDot : 十字の形に描画する
      ******************************************/

      /*
        CrossDot::operator() : draw 上の c を中心に十字の形に描画する

        中央は描画色、周囲は背景色で描画する。但し、前回描画した位置が隣であった場合、
        その位置に応じて重なりあうドットは描画しない。
      */
      bool CrossDot::operator()( DrawingArea_IF& draw, Coord< int > c )
      {
        draw.pset( c, fg_ );
        if ( isFirst_ || ( ! ( pre_.x + 1 == c.x && pre_.y == c.y ) ) ) // 前回の右側ではない
          if ( c.x - 1 >= 0 ) draw.pset( Coord< int >( c.x - 1, c.y ), bg_ );
        if ( isFirst_ || ( ! ( pre_.x - 1 == c.x && pre_.y == c.y ) ) ) // 前回の左側ではない
          if ( c.x + 1 < draw.size().x ) draw.pset( Coord< int >( c.x + 1, c.y ), bg_ );
        if ( isFirst_ || ( ! ( pre_.x == c.x && pre_.y + 1 == c.y ) ) ) // 前回の下側ではない
          if ( c.y - 1 >= 0 ) draw.pset( Coord< int >( c.x, c.y - 1 ), bg_ );
        if ( isFirst_ || ( ! ( pre_.x == c.x && pre_.y - 1 == c.y ) ) ) // 前回の上側ではない
          if ( c.y + 1 < draw.size().y ) draw.pset( Coord< int >( c.x, c.y + 1 ), bg_ );

        pre_ = c;
        isFirst_ = false;

        return( true );
      }

    } // namespace pen

  } // namespace gl

} // namespace tadah_fussy
